// Definition of KASSERT() macro, and other useful debug macros
// Copyright (c) 2001, David H. Hovemeyer <daveho@cs.umd.edu>
// $Revision: 1.4 $

// This is free software.  You are permitted to use,
// redistribute, and modify it as specified in the file "COPYING".

#ifndef KASSERT_H
#define KASSERT_H

#include "screen.h"

#ifndef NDEBUG

#define KASSERT( cond ) 				\
do {							\
    if ( !(cond) ) {					\
	Set_Current_Attr( ATTRIB(RED, GRAY|BRIGHT) );	\
	Print( "Failed assertion: %s at %s, line %d",	\
		#cond, __FILE__, __LINE__ );		\
	while (1)					\
	   ; 						\
    }							\
} while (0)

// Spin for some number of iterations.
// This is useful for slowing down things that go by too
// quickly.
#define PAUSE( count )			\
do {					\
    unsigned long i;			\
    for ( i = 0; i < (count); ++i )	\
	;				\
} while (0)

// Stop dead.
#define STOP() while (1)

#else

// The debug macros are no-ops when NDEBUG is defined.
#define KASSERT( cond )
#define PAUSE( count )
#define STOP()

#endif

#endif // KASSERT_H
