// Kernel threads
// Copyright (c) 2001, David H. Hovemeyer <daveho@cs.umd.edu>
// $Revision: 1.6 $

// This is free software.  You are permitted to use,
// redistribute, and modify it as specified in the file "COPYING".

#ifndef KTHREAD_H
#define KTHREAD_H

#include "ktypes.h"

// Register state.
struct Registers {
    unsigned long eax;	// 0
    unsigned long ebx;	// 4
    unsigned long ecx;	// 8
    unsigned long edx;	// 12
    unsigned long esi;	// 16
    unsigned long edi;	// 20
    unsigned long ebp;	// 24
    unsigned long esp;	// 28
#if 0
    unsigned long eip;	// 32
#endif
};

// Kernel thread context data structure.
struct Kernel_Thread {
    struct Registers regs;
    struct Kernel_Thread* next;
    void* stackPage;
};

// Thread start functions should have this signature.
typedef void (*Thread_Start_Func)( unsigned long arg );

// Queue of threads.  We use this data structure and its
// operations for several purposes:
//  - the run queue
//  - wait queues (where threads wait for a condition or event)
//  - the reaper queue (where completed threads are sent)
struct Thread_Queue {
    struct Kernel_Thread *head, *tail;
};
#define THREAD_QUEUE_INITIALIZER { 0, 0 }

// Scheduler operations.
void Init_Scheduler( void );
Boolean Start_Thread( Thread_Start_Func startFunc, unsigned long arg );
void Make_Runnable( struct Kernel_Thread* kthread );
void Make_Runnable_Atomic( struct Kernel_Thread* kthread );
struct Kernel_Thread* Get_Current( void );
void Schedule( void );
void Yield( void );

// General thread queue functions.
void Enqueue_Thread( struct Thread_Queue* queue, struct Kernel_Thread* kthread );
struct Kernel_Thread* Dequeue_Thread( struct Thread_Queue* queue );
Boolean Is_Thread_Queue_Empty( struct Thread_Queue* queue );
void Make_Thread_Queue_Empty( struct Thread_Queue* queue );

// Wait queue functions.
void Wait( struct Thread_Queue* waitQueue );
void Wake_Up( struct Thread_Queue* waitQueue );

#endif // KTHREAD_H
