// GeekOS C code entry point
// Copyright (c) 2001, David H. Hovemeyer <daveho@cs.umd.edu>
// $Revision: 1.17 $

// This is free software.  You are permitted to use,
// redistribute, and modify it as specified in the file "COPYING".

#include "bootinfo.h"
#include "memops.h"
#include "screen.h"
#include "mem.h"
#include "int.h"
#include "kthread.h"
#include "keyboard.h"

static void Init_BSS( void );
static void Silly( unsigned long arg );

void Main( struct Boot_Info* bootInfo )
{
    Init_BSS();
    Init_Screen();
    Init_Mem( bootInfo );
    Init_Interrupts();
    Init_Scheduler();
    Init_Keyboard();

    Set_Current_Attr( ATTRIB(BLACK, CYAN|BRIGHT) );
    Print( "Welcome to GeekOS!\n" );

    Start_Thread( Silly, 0xdeadbeef );

    // For now, we're just a glorified typewriter
    while ( TRUE ) {
	Keycode keycode = Wait_For_Key();
	if ( !(keycode & KEY_SPECIAL_FLAG) && !(keycode & KEY_RELEASE_FLAG) ) {
	    keycode &= 0xff;
	    Print( "%c", (keycode == '\r') ? '\n' : keycode );
	}
    }
}

// Initialize the .bss section of the executable image.
static void Init_BSS( void )
{
    extern char __bss_start;
    extern char end;

    // Fill .bss with zeroes
    Fill_Memory( &__bss_start, '\0', &end -  &__bss_start );
}

// This thread just tests that threads can exit, and that the
// reaper thread collects their memory.
static void Silly( unsigned long arg )
{
    Print( "Silly thread! %x\n", arg );
}
