// Memory handling and allocation
// Copyright (c) 2001, David H. Hovemeyer <daveho@cs.umd.edu>
// $Revision: 1.9 $

// This is free software.  You are permitted to use,
// redistribute, and modify it as specified in the file "COPYING".

#ifndef MEM_H
#define MEM_H

#include "ktypes.h"

struct Boot_Info;

// Page flags
#define PAGE_KERN      0x0001	// page used by kernel code or data
#define PAGE_HW        0x0002	// page used by hardware (e.g., ISA hole)
#define PAGE_ALLOCATED 0x0004	// page is allocated
#define PAGE_UNUSED    0x0008	// page is unused

// x86 has 4096 byte pages
#define PAGE_POWER 12
#define PAGE_SIZE (1<<PAGE_POWER)
#define PAGE_MASK (~(0xffffffff << PAGE_POWER))

// PC memory map
#define ISA_HOLE_START 0x0A0000
#define ISA_HOLE_END   0x100000

// Each page of physical memory has one of these structures
// associated with it, to do allocation and bookkeeping.
struct Page {
    unsigned flags;
    unsigned long nextFree;
};

void Init_Mem( struct Boot_Info* bootInfo );
void* Alloc_Page( void );
void* Alloc_Page_Atomic( void );
void Free_Page( void* pageAddr );
void Free_Page_Atomic( void* pageAddr );

// FIXME: should inline these (?)
Boolean Is_Page_Multiple( unsigned long addr );
unsigned long Round_Up_To_Page( unsigned long addr );

#define Page_Index(addr) ((unsigned long)addr >> PAGE_POWER)

#endif // MEM_H
