; Thread context switch.
; Copyright (c) 2001, David H. Hovemeyer <daveho@cs.umd.edu>
; $Revision: 1.6 $

; This is free software.  You are permitted to use,
; redistribute, and modify it as specified in the file "COPYING".

[BITS 32]

[EXTERN g_currentThread]

[GLOBAL Switch_To_Thread]
[GLOBAL Restore_Thread]

; ----------------------------------------------------------------------
; Switch_To_Thread()
;   Save context of currently executing thread, and activate
;   the thread whose context object is passed as a parameter.
; 
; Parameter: 
;   - ptr to Kernel_Thread whose state should be restored and made active
;
; Notes:
; Called with interrupts disabled.
; This must be kept up to date with definition of Kernel_Thread
; struct, in kthread.h.
; ----------------------------------------------------------------------
align 16
Switch_To_Thread:
	; Save eax, because we're going to use it
	; to point to current thread context obj
	push	eax

	; Save state of current thread.
	mov	eax, [g_currentThread]
	mov	[eax+4], ebx
	pop	ebx		; get saved value of eax
	mov	[eax+0], ebx	; put saved eax in thread context obj
	mov	[eax+8], ecx
	mov	[eax+12], edx
	mov	[eax+16], esi
	mov	[eax+20], edi
	mov	[eax+24], ebp
	mov	[eax+28], esp

	; Get the address of the new thread context object
	mov	eax, [esp+4]

	; Jump into Restore_Thread, which will activate the
	; thread context pointed to by eax
	jmp	Restore_Thread


; ----------------------------------------------------------------------
; Restore_Thread()
;   Activate a thread whose thread context object is pointed to
;   by the eax register.
;
; Notes:
; The context to be switched to is passed in a register
; because there may not be a valid stack; for example, see
; Shutdown_Thread().
; As with Switch_To_Thread(), this function must be kept
; up to date with the Kernel_Thread struct in kthread.h.
; ----------------------------------------------------------------------
align 16
Restore_Thread:
	; Restore state of thread being switched to.
	mov	[g_currentThread], eax
	mov	ebx, [eax+4]
	mov	ecx, [eax+8]
	mov	edx, [eax+12]
	mov	esi, [eax+16]
	mov	edi, [eax+20]
	mov	ebp, [eax+24]
	mov	esp, [eax+28]
	mov	eax, [eax+0]	; now we can restore eax

	; We'll return to the place where the thread was
	; executing last.
	ret
